@@ -50,6 +50,7 @@ enum program_args {
     ARG_RING_BUFFER_COUNT,
     ARG_RING_BUFFER_SIZE,
     ARG_VERBOSE,
+    ARG_AMQP_BLOCK,
     ARG_HELP
 };
 
@@ -101,6 +102,10 @@ struct option_info option_info[] = {
      "",
      "Print extra info, multiple instance increase verbosity.",
      ""},
+    {{"amqp_block", no_argument, 0, ARG_AMQP_BLOCK},
+     "",
+     "Stop reading incoming messages if the buffer is full (%s)",
+     DEFAULT_AMQP_BLOCK},
     {{"help", no_argument, 0, ARG_HELP}, "", "Print help.", ""}};
 
 static void usage(char *program) {
@@ -191,6 +196,7 @@ int main(int argc, char **argv) {
     app.peer_port = DEFAULT_INET_PORT;
     app.ring_buffer_size = atoi(DEFAULT_RING_BUFFER_SIZE);
     app.ring_buffer_count = atoi(DEFAULT_RING_BUFFER_COUNT);
+    app.amqp_block = false; /* disabled */
 
     int num_args = sizeof(option_info) / sizeof(struct option_info);
     struct option *longopts = malloc(sizeof(struct option) * num_args);
@@ -251,6 +257,9 @@ int main(int argc, char **argv) {
         case 'v':
             app.verbose++;
             break;
+        case ARG_AMQP_BLOCK:
+            app.amqp_block = true;
+            break;
         case 'h':
         case ARG_HELP:
             usage(argv[0]);
@@ -265,27 +274,28 @@ int main(int argc, char **argv) {
     memset(matches, 0, sizeof(matches));
 
     match_regex(AMQP_URL_REGEX, matches, 10, app.amqp_con.url);
-    if (matches[2] != NULL) {
+    if (matches[3] != NULL) {
         app.amqp_con.user = strdup(matches[2]);
     }
-    if (matches[4] != NULL) {
+    if (matches[5] != NULL) {
         app.amqp_con.password = strdup(matches[4]);
     }
-    if (matches[5] == NULL || matches[8] == NULL) {
+    if (matches[6] == NULL || matches[9] == NULL) {
         fprintf(stderr, "Invalid AMQP URL: %s", app.amqp_con.url);
         exit(1);
     }
-    app.amqp_con.host = strdup(matches[5]);
-    app.amqp_con.address = strdup(matches[8]);
-    if (matches[7] != NULL) {
-        app.amqp_con.port = strdup(matches[7]);
+    app.amqp_con.host = strdup(matches[6]);
+    app.amqp_con.address = strdup(matches[9]);
+    if (matches[8] != NULL) {
+        app.amqp_con.port = strdup(matches[8]);
     }
 
     if (app.standalone) {
         printf("Standalone mode\n");
     }
 
-    app.rbin = rb_alloc(app.ring_buffer_count, app.ring_buffer_size);
+    app.rbin =
+        rb_alloc(app.ring_buffer_count, app.ring_buffer_size, app.amqp_block);
 
     app.amqp_rcv_th_running = true;
     pthread_create(&app.amqp_rcv_th, NULL, amqp_rcv_th, (void *)&app);
@@ -303,14 +313,15 @@ int main(int argc, char **argv) {
         sleep(1);
         if (sleep_count == app.stat_period) {
             printf("in: %ld(%ld), amqp_overrun: %ld(%ld), out: %ld(%ld), "
-                   "sock_overrun: %ld(%ld), batch_size: %f\n",
+                   "sock_overrun: %ld(%ld), link_credit: %f\n",
                    app.amqp_received, app.amqp_received - last_amqp_received,
                    app.rbin->overruns, app.rbin->overruns - last_overrun,
                    app.sock_sent, app.sock_sent - last_out,
                    app.sock_would_block,
                    app.sock_would_block - last_sock_overrun,
-                   app.amqp_received / (float)app.amqp_total_batches);
-            sleep_count = 0;
+                   app.link_credit / (float)app.amqp_received);
+
+            sleep_count = 1;
         }
         sleep_count++;
         last_amqp_received = app.amqp_received;
